import logging
import os
import config

if not os.path.exists(config.log_directory):
    os.makedirs(config.log_directory)
logging.basicConfig(filename=config.log_file, level=logging.DEBUG, format=config.log_format)

def new_log(bug_type, message):
    """
    Crée un logging correspondant au type de bug spécifié avec le message donné.

    Args:
        bug_type (str): Le type de bug (DEBUG, INFO, WARNING, ERROR, CRITICAL).
        message (str): Le message à enregistrer dans le logging.
    """
    bug_type_mapping = {
        "DEBUG": logging.debug,
        "INFO": logging.info,
        "WARNING": logging.warning,
        "ERROR": logging.error,
        "CRITICAL": logging.critical
    }

    if bug_type not in bug_type_mapping:
        raise ValueError(f"Type de bug invalide : {bug_type}")

    bug_type_mapping[bug_type](message)






import csv
from dataclasses import dataclass
from datetime import datetime, timezone
import json

from sqlalchemy import Column, String, Integer, create_engine, DateTime, and_, text, desc
from sqlalchemy.exc import OperationalError
from sqlalchemy.orm import sessionmaker, declarative_base

import config
from synch2jira.log import new_log
from synch2jira.issue_S2 import IssueS2
from synch2jira.issue_csv import IssueCSV
from synch2jira.issue_json import IssueJSON

Base = declarative_base()
engine = create_engine(config.workflow_database_file)
Session = sessionmaker(bind=engine)
session = Session()


@dataclass
class IssueWorkflow(Base):
    __tablename__ = 'issue_worflow'
    id = Column(Integer, autoincrement=True, primary_key=True)
    issueKey = Column(String)
    status = Column(String)
    updated = Column(DateTime)
    from_time = Column(String)
    to_time = Column(String)

    def __init__(self, issueKey, status, updated, from_time, to_time):
        self.from_time = from_time
        self.issueKey = issueKey
        self.updated = updated
        self.status = status
        self.to_time = to_time

    @staticmethod
    def get_workflow_history(self):
        return self.all()

    @staticmethod
    def update_last_updated(issue_key, updated):
        issues = IssueWorkflow.find_by_id(issue_key)
        for issue in issues:
           issue.updated = updated
           issue.update()
        return True

    @staticmethod
    def fill_issue_workflow_bdd():
        try : 
            IssueWorkflow.create_all_tables()
            #IssueWorkflow.delete_all()
            IssueWorkflow.get_workflow()
            print("fin de la creation ")

        except Exception as e :
            print("ERROR",f"Erreur lors du remplissage de la base de données : {str(e)}")
            new_log("ERROR",f"Erreur lors du remplissage de la base de données : {str(e)}")

    @staticmethod
    def save_issue_workflow(issue_key):
        try:
            jira_transitions = config.jiraStatusName
            for status in jira_transitions:
                from_time, to_time = IssueWorkflow.get_workflow(issue_key, status)  # 691ms
                workflow = IssueWorkflow(issue_key, status, datetime.now(), from_time, to_time)
                workflow.save()  # 7ms
        except Exception as e:
            new_log("ERROR",f"Erreur lors de la sauvegarde du workflow de l'issue {issue_key} : {str(e)}")

    @staticmethod
    def did_issue_have_state(issue_key, state):
        workflow_list = IssueWorkflow.find_by(issueKey=issue_key, status=state)
        for workflow in workflow_list:
            if workflow.to_time:
                return True
        return False

    @staticmethod
    def synchronize_data_with_jira():
        print('calling synch')
        jira_columns = config.jiraStatusName
        issue_in_jira_list = IssueS2.all_lite()
        workflow_list = IssueWorkflow.all_keys()
        for issue_jira in issue_in_jira_list:
            if issue_jira.key not in workflow_list:
                IssueWorkflow.save_issue_workflow(issue_jira.key)
            else:
                print(IssueWorkflow.find_by_id(issue_jira.key))
                workflow = IssueWorkflow.find_by_id(issue_jira.key)[0]
                print(issue_jira.key, workflow.updated, issue_jira.status_updated)
                # print(type(workflow.updated),type(issue_jira.updated))
                is_updated_on_date = IssueWorkflow.compare_str_dates(workflow.updated, issue_jira.status_updated)
                if not is_updated_on_date:
                    for status in jira_columns:
                        from_time, to_time = IssueWorkflow.get_workflow(issue_jira.key, status)
                        workflow = IssueWorkflow.find_by(issueKey=issue_jira.key, status=status)[0]
                        workflow.from_time = from_time
                        workflow.to_time = to_time
                        workflow.update()
                        #print(workflow.issueKey)
                    # workflow.save()

    @staticmethod
    def compare_str_dates(date1, date2):
        date1 = date1.replace(tzinfo=timezone.utc)
        date2 = datetime.strptime(date2, "%Y-%m-%dT%H:%M:%S.%f%z").replace(tzinfo=timezone.utc)
        #print(f'date 1 {date1} , date 2 {date2}')
        if date1 > date2:
            # KAN-17 2024-03-05 09:13:40.998417 2024-03-05T11:27:28.959+0100
            # KAN-99 2024-03-05 11:06:19.485890 2024-03-05T11:27:29.733+0100
            return True

        return False

    @staticmethod
    def get_workflow():
        workflows = []
        #issues = IssueS2.get_issue_data_from_jira()
        issues = IssueS2.get_issue_list_from_json()
        try : 
            for issue in issues:
                issue_key = issue["key"]
                changelog = issue.get("changelog", {})
                histories = changelog.get("histories", [])
                for stat in config.jiraStatusName:
                    from_time = []
                    to_time = []
                    for history in histories:
                        created = history.get("created", "")
                        items = history.get('items', [])
                        for item in items:
                            field = item['field']
                            if field == "status":
                                if item['fromString'] == stat:
                                    from_time.append(created)
                                if item['toString'] == stat:
                                    to_time.append(created)
                    min_from_time = None if from_time == [] else min(from_time)
                    max_to_time = None if to_time == [] else max(to_time)
                    workflow = IssueWorkflow(issue_key, stat, datetime.now(), min_from_time, max_to_time)
                    # print((issue_key, stat, min_from_time, max_to_time))
                    workflows.append(workflow)
                    new_log("DEBUG",f"Liste des from_time : {from_time}")
                    new_log("DEBUG",f"Liste des to_time : {to_time}")
                    #print("DEBUG",f"Liste des from_time : {from_time}")
                    #print("DEBUG",f"Liste des to_time : {to_time}")
                    new_log("DEBUG",f"Liste des workflow : {workflows}")
                
        except Exception as e: 
            new_log("ERROR",f"Erreur lor du parsage dans la fonction get_workflow : {str(e)}")
        IssueWorkflow.save_all(workflows)

    def get_workflow_():
        pass

    @staticmethod
    def save_result_in_csv(result, entete):
        try: 
            filename = config.csv_file
            with open(filename, mode="w", newline="") as file:
                writer = csv.writer(file)
                writer.writerow(entete)
                for row in result:
                    writer.writerow(row)
            new_log("INFO",f"Les données ont été écrites dans {filename}")
        except Exception as e: 
            new_log("ERROR","Erreur lors de la creation du fichier workflow csv")
    
    def generate_workflow_data_in_csv():
        with open(config.json_issues_file,'r') as file :
           data = json.load(file)
           issues = [IssueJSON.json_to_issue(issue_data,data) for issue_data in data]
           IssueCSV.generate_csv_data(issues,'data.csv')


    @staticmethod
    def create(**kwargs):  # avec les argument on creee un objet et on le save
        workflow = IssueWorkflow(**kwargs)
        session.add(workflow)
        session.commit()
        return workflow

    @staticmethod
    def create_all_tables():
        try:
            Base.metadata.create_all(engine)
            return True
        except OperationalError:
            return new_log("ERROR","Les tables existent déjà.")
        except Exception as e:
            return new_log("ERROR",f"Une erreur s'est produite : {e}")

    @staticmethod
    def new(**kwargs):  # avec les argument on creee un objet sans le save
        return IssueWorkflow(**kwargs)

    @staticmethod
    def all():
        return session.query(IssueWorkflow).all()

    @staticmethod
    def all_keys():
        return list(set([workflow.issueKey for workflow in IssueWorkflow.all()]))

    @staticmethod
    def length():
        return len(IssueWorkflow.all())

    def save(self):
        session.add(self)
        session.commit()
        return self

    def get(self):
        return session.query(IssueWorkflow).filter(
            and_(IssueWorkflow.issueKey == self.issueKey, IssueWorkflow.status == self.status)).first()

    @staticmethod
    def first():
        return session.query(IssueWorkflow).first()

    @staticmethod
    def last():
        return session.query(IssueWorkflow).order_by(desc(IssueWorkflow.issueKey)).first()

    @staticmethod
    def find_by(**kwargs):
        return session.query(IssueWorkflow).filter_by(**kwargs).all()

    @staticmethod
    def find_by_id(key):
        return session.query(IssueWorkflow).filter_by(issueKey=key).all()

    @staticmethod
    def update(instance, **kwargs):
        for key, value in kwargs.items():
            setattr(instance, key, value)
        session.commit()

    def update(self):
        self.updated = datetime.now()
        session.merge(self)
        session.commit()

    @staticmethod
    def update_all(**kwargs):
        session.query(IssueWorkflow).update(kwargs)
        session.commit()

    def delete(self):
        session.delete(self)
        session.commit()

    @staticmethod
    def delete_all():
        session.execute(text("DELETE FROM issue_worflow"))
        session.commit()

    @staticmethod
    def save_all(liste):
        try:
            session.add_all(liste)
            session.commit()
        except Exception as e:
            new_log("ERROR",f" eErreur lors de la sauvegarde des workflow deans la BDD: {e}")
            session.rollback()



import json
import os
import threading

import requests

import config
from synch2jira.issue import Issue
from synch2jira.log import new_log
from synch2jira.issue_S1 import IssueS1


class IssueS2Thread(threading.Thread):
    def __init__(self, start_at, batch_size, fields, jql_query, name):
        threading.Thread.__init__(self, name=name)
        self.start_at = start_at
        self.batch_size = batch_size
        self.fields = fields
        self.jql_query = jql_query
        self.issues = []

    def run(self):
        params = {'jql': self.jql_query, "startAt": self.start_at, "maxResults": self.batch_size, "fields": self.fields,
                  "expand": 'changelog'}
        response = IssueS2.jira_request(method='get', url=config.jira_url_all, params=params)
        if response.status_code == 200:
            batch_tickets = response.json()["issues"]
            # print(self.name,self.batch_size)
            self.issues.extend(batch_tickets)


class IssueS2(Issue):
    key: str

    def __init__(self, key=None, summary=None, description=None, updated=None, status_updated=None, status=None):
        Issue.__init__(self, summary, description, updated, status, status_updated)
        self.key = key
        # if self.miror is not None:
        #     self.miror.miror = self

    def format_issue_into_json(self):
        issue = {
            "fields": {
                "description": {
                    "content": [
                        {
                            "content": [
                                {
                                    "text": self.description if self.description else "",
                                    "type": "text"
                                }
                            ],
                            "type": "paragraph"
                        }
                    ],
                    "type": "doc",
                    "version": 1
                },
                "issuetype": {
                    "id": config.key_issue_type
                },
                "project": {
                    "id": config.project_key
                },
                "labels": config.labels if config.labels else [],
                "summary": self.summary
            },
            "update": {},
        }
        if self.miror is not None:
            issue['fields'][config.s1_id_in_jira] = str(self.miror.id)
        return issue

    @staticmethod
    def convertir_json_en_issue(issue_json):
        try:
            issue_key = issue_json.get('key', None)
            issue_summary = issue_json['fields'].get('summary', None)
            description_data = issue_json['fields'].get('description', None)

            description_text = None
            if description_data is not None and 'content' in description_data:
                content_list = description_data['content']

                if content_list and content_list[0].get('content'):
                    description_text = content_list[0]['content'][0].get('text', None)

            issue_updated = issue_json['fields'].get('updated', None)
            status = issue_json['fields']['status']['name']
            status_updated = issue_json['fields'].get('statuscategorychangedate', None)
            return IssueS2(key=issue_key, summary=issue_summary, description=description_text,
                                         updated=issue_updated, status_updated=status_updated,
                                         status=status)
        except Exception as e:
            new_log("ERROR", f"Erreur lors de la conversion de json en IssueS2 : {str(e)}")
            return None

    @classmethod
    def create_issue_lite(cls, issue_json):
        issue = IssueS2()
        issue.key = issue_json['key']
        issue.updated = issue_json['updated']
        return issue

    @staticmethod
    def get_miror(self):
        response = IssueS2.jira_request("GET", config.jira_url_ticket + self.key)
        if response.status_code == 200:
            mirror_id = response.json()['fields'].get(config.s1_id_in_jira, None)
            miror = IssueS1.find_by_id(mirror_id)
            return miror

    def get(self):
        try:
            response = IssueS2.jira_request("GET", config.jira_url_ticket + self.key)
            json_data = response.json()
            issue = IssueS2.convertir_json_en_issue(json_data)
            if issue is not None:
                return IssueS2(key=issue.key, summary=issue.summary, description=issue.description,
                                             status_updated=issue.status,
                                             updated=issue.updated, status=issue.status)
        except Exception as e:
            return e

    @staticmethod
    def jira_request(method, url, auth=config.auth, headers={}, params={}, data=None):
        try:
            response = requests.request(method, url, auth=auth, headers=headers, params=params, json=data,
                                        verify=config.verify_ssl_certificate)
            response.raise_for_status()  # Lève une exception pour les codes d'erreur HTTP
            return response
        except requests.RequestException as e:
            new_log("ERROR", f"Erreur lors de la requête HTTP : {str(e)}")
            return

    @staticmethod
    def find_by_id(key):
        try:
            # response = requests.request("GET", config.jira_url_ticket + key, headers={}, auth=config.auth)
            response = IssueS2.jira_request(method='get', url=config.jira_url_ticket + key)
            json_data = response.json()
            issue = IssueS2.convertir_json_en_issue(json_data)
            if issue is not None:
                return IssueS2(key=issue.key, summary=issue.summary, description=issue.description,
                                             updated=issue.updated, status=issue.status)
        except Exception as e:
            new_log("ERROR", f"Erreur lors de la recherche de la clé : {str(e)}")
            return e

    @staticmethod
    def find_by(jql_query):
        return IssueS2.get_issue_list_from_jira(jql_query=jql_query)

    # @staticmethod
    # def delete_all(jql_query):
    #     payload = {"jql": jql_query}
    #     response = requests.get(configurations.jira_url_all, params=payload, headers=configurations.headers, auth=configurations.auth)
    #     if response.status_code == 200:
    #         issues = response.json().get("issues", [])
    #         for issue in issues:
    #             IssueS2.delete(issue["key"])
    #     else:
    #         return f"Failed to execute search query. Status code: {response.status_code}"

    @staticmethod
    def get_jira_transitions(issue_key):
        transitions = []
        transition_url = config.jira_url_ticket + issue_key + "/transitions"
        # transitions_response = requests.get(transition_url, auth=config.auth)
        response = IssueS2.jira_request(method='get', url=transition_url)
        transitions_data = response.json()
        transitions = [transition['name'] for transition in transitions_data['transitions']]
        return transitions

    @staticmethod
    def get_jira_status(projectIdOrKey=config.projectIdOrKey):
        status_url = f"{config.jira_url_base}/rest/api/3/project/{projectIdOrKey}/statuses"
        response = IssueS2.jira_request(method='get', url=status_url)
        if response.status_code == 200:
            json_data = response.json()[0]
            status_names = set()
            for status in json_data["statuses"]:
                status_names.add(status["name"])
            return status_names
        else:
            return response.status_code

    @staticmethod
    def length():
        issues = IssueS2.all()
        return len(issues)

    @staticmethod
    def all():
        # return IssueS2.find_by(config.jql_query)
        return IssueS2.get_issue_list_from_json()

    @staticmethod
    def all_lite():
        fields = ['updated', 'statuscategorychangedate']
        # issues = IssueS2.get_issue_list_from_jira(fields)
        issues = IssueS2.get_issue_list_from_json()
        return [IssueS2(key=issue["key"], updated=issue["fields"].get("updated", None),
                                      status_updated=issue["fields"].get("statuscategorychangedate")) for issue in
                issues]

    @staticmethod
    def all_weight():
        fields = ["updated", 'statuscategorychangedate', 'summary', 'description']
        # issues = IssueS2.get_issue_list_from_jira(fields)
        issues = IssueS2.get_issue_list_from_json()
        return [IssueS2.convertir_json_en_issue(issue) for issue in issues]

    # take a list of fiels as parameter : fiels you wwant to get in the response
    @staticmethod
    def get_issue_list_from_jira(fields='', jql_query=config.jql_query):
        try:
            batch_size = 100
            issues = []
            start_at = 0
            while True:
                params = {'jql': jql_query, "startAt": start_at, "maxResults": batch_size, "fields": fields}
                response = IssueS2.jira_request(method='get', url=config.jira_url_all, params=params)
                if response.status_code == 200:
                    batch_tickets = response.json()["issues"]
                    issues.extend(batch_tickets)
                    start_at += batch_size
                    if len(batch_tickets) < batch_size:
                        break  # on arrete la boucle si moins de tickets sont retournés que le batch_size
                else:
                    return []
            return issues
        except Exception as e:
            new_log("ERROR", f"Erreur lors de la recherche des tickets sur jira : {str(e)}")

    @staticmethod
    def get_issue_data_from_jira(fields='', jql_query=config.jql_query):
        try:
            batch_size = 100
            issues = []
            start_at = 0
            while True:
                params = {'jql': jql_query, "startAt": start_at, "maxResults": batch_size, "fields": fields,
                          "expand": 'changelog'}
                response = IssueS2.jira_request(method='get', url=config.jira_url_all, params=params)
                if response.status_code == 200:
                    print(response.json().keys())
                    print(response.json()['total'])

                    batch_tickets = response.json()["issues"]
                    issues.extend(batch_tickets)
                    start_at += batch_size
                    if len(batch_tickets) < batch_size:
                        break  # on arrete la boucle si moins de tickets sont retournés que le batch_size
                else:
                    return []
            IssueS2.save_json_file(config.json_issues_file, issues)
            return issues
        except Exception as e:
            new_log("ERROR", f"Erreur lors de la recuperation des données avec changeLog depuis jira : {str(e)}")

    @staticmethod
    def get_issue_data_from_jira_with_thread(fields=config.fields_to_use, jql_query=config.jql_query):
        try:
            batch_size = 100
            start_at = 0
            total_issues = IssueS2.get_number_of_jira_tickets()
            threads = []
            thread_id = 0
            issues = []

            while start_at < total_issues:
                name = f'Thread_{thread_id}'
                thread = IssueS2Thread(start_at, batch_size, fields, jql_query, name)
                threads.append(thread)
                thread.start()
                start_at += batch_size
                thread_id += 1

            for thread in threads:
                thread.join()
                issues.extend(thread.issues)

            IssueS2.save_json_file(config.json_issues_file, issues)
            # for issue in issues:
            #     histories = issue["changelog"].get('histories',[])
            #     print(issue["key"],len(histories))

            return issues
        except Exception as e:
            new_log("ERROR", f"Erreur lors de la récupération des données depuis JIRA : {str(e)}")
            return []

    @staticmethod
    def get_number_of_jira_tickets():
        try:
            params = {'jql': config.jql_query, "fields": ['summary']}
            response = IssueS2.jira_request(method='get', url=config.jira_url_all, params=params)
            if response.status_code == 200:
                print(response.json())
                return response.json()['total']
        except Exception as e:
            new_log("ERROR", f"Erreur lors de la recuperation du nombre de ticket depuis jira depuis jira : {str(e)}")

    @staticmethod
    def get_project_list():
        project_url = f'{config.jira_url_base}/rest/api/3/project'
        response = IssueS2.jira_request(method='get', url=project_url)
        if response.status_code == 200:
            projects = response.json()

            for project in projects:
                print(project)
                print(project['key'], '-', project['name'], '-', project['id'])
            return [[project['key'], project['name'], project['id']] for project in projects]
        else:
            new_log("ERROR", f"La requête a échoué avec le code d'erreur {response.status_code}")

    def save(self):
        response = IssueS2.jira_request(method='POST', url=config.jira_url_ticket,
                                                      data=self.format_issue_into_json())

        if response.status_code == 201:
            # return (json.loads(response.text)).get("key")
            return response
        else:
            new_log("ERROR", f"Erreur lors de la création du ticket. Code d'erreur : {response.status_code}")

            # def delete(self):

    #     response = requests.request("DELETE", config.jira_url_ticket + self.key, auth=config.auth)
    #
    #     if response.status_code == 204:
    #         print(f"Suppression effectuée avec succées")
    #     else:
    #         print(f"Erreur lors de la suppression du ticket. Code d'erreur : {response.status_code}")

    def update(self):
        response = IssueS2.jira_request(method='put', url=config.jira_url_ticket + self.key,
                                                      data=self.format_issue_into_json())
        return response.text

    # takes transitions as argument not status Name (Columns)
    def change_issue_status(self, new_status):
        transition_url = config.jira_url_ticket + self.key + "/transitions"
        # response = requests.get(transition_url, auth=config.auth)
        response = IssueS2.jira_request(method='get', url=transition_url)
        transitions = response.json()['transitions']
        transition_id = None
        for transition in transitions:
            if transition['name'] == new_status:
                transition_id = transition['id']
                break
        if not transition_id:
            return None

        data = {
            "transition": {"id": transition_id}
        }
        headers = {'Content-Type': 'application/json'}

        response = requests.post(transition_url, json=data, headers=headers, auth=config.auth)

        if response.status_code == 204:
            return transition_id
        else:
            new_log("ERROR", "Erreur lors du changement de statut:{response.text}")
            return None

    # @staticmethod
    # def supprimer_tickets_entre_a_and_b(a, b):
    #     for i in range(a, b):
    #         key = "KAN-" + str(i)
    #         issue_to_delete = IssueS2(key=key, summary="", description="", updated="", status="")
    #         issue_to_delete.delete()

    @staticmethod  ####### 
    def check_connection(jira_url=config.jira_url_base, username=config.username, api_token=config.api_token):
        try:
            response = IssueS2.jira_request('head', jira_url)
            if response.status_code == 200:
                return True
            else:
                return False
        except requests.RequestException as e:
            new_log("CRITICAL", f"Erreur de connexion à Jira: {e}")
            return False

    @staticmethod
    def create_isssue_and_move_to_done_for_performance_test(a, b):
        new_status = 'Terminé'
        status = "Pret"
        key_list = []
        for i in range(a, b):
            description = f'description_{i}'
            summary = f'summary_{i}'
            issue = IssueS2(summary=summary, description=description)
            key = issue.save()
            print(key)
            # key_list.append(key)
            # issue = IssueS2.find_by_id(key)
            # result = issue.change_issue_status(new_status)
            # print(f'transition id {result}   key {key}  iteration {i}')
            # issue.delete()
            # break
        # print(key_list)
        # return key_list

    @staticmethod
    def create_issues(a, b):
        for i in range(a, b):
            description = f'description_{i}'
            summary = f'summary_{i}'
            issue = IssueS2(summary=summary, description=description)
            key = issue.save()
            print(key)

    @staticmethod
    def all_key():
        fields = ["updated"]
        # return [issue.get('key') for issue in IssueS2.get_issue_list_from_jira(fields)]
        return [issue.get('key') for issue in IssueS2.get_issue_list_from_json()]

    @staticmethod
    def save_data_json_in_file():
        all_tickets = IssueS2.get_issue_data_from_jira()
        IssueS2.save_json_file(config.json_issues_file, all_tickets)

    @staticmethod
    def save_json_file(file_name, data):
        try:
            if not os.path.exists(config.files_directory):
                new_log("WARNING", f"Creation du dossier {config.files_directory}")
                os.makedirs(config.files_directory)
            with open(file_name, 'w') as json_file:
                json.dump(data, json_file, indent=4)
            new_log("INFO", f"Les données ont été ecrite avec succes dans {config.json_issues_file}")
            return True
        except Exception as e:
            new_log("ERROR", f"Erreur lors de la creation du fichier json_issues_file : {str(e)}")
            return False

    @staticmethod
    def get_issue_list_from_json():
        try:
            with open(config.json_issues_file, 'r') as file:
                data = json.load(file)
            return data
        except Exception as e:
            new_log("ERROR", f"Erreur lors de la lecture du fichier json_issues_file : {str(e)}")



import json
import os

import matplotlib
import matplotlib.pyplot as plt
import pandas as pd

import config
from synch2jira.issue_csv import IssueCSV
from synch2jira.issue_json import IssueJSON

matplotlib.use('Agg')


def generate_issue_data_mining_csv(state1=config.workflow_status1, state2=config.workflow_status2):
    with open(config.json_issues_file, 'r') as file:
        json_data = json.load(file)
    issue_list = [IssueJSON.json_to_issue(issue_data, json_data, state1, state2) for issue_data in
                  json_data]
    IssueCSV.generate_csv_data(issue_list, config.csv_issue_file)


def csv_to_dataframe(file_path):
    df = pd.read_csv(file_path)
    return df


def str_df_to_dt_df(dataframe, use_workflow=config.use_workflow):
    if use_workflow:
        dataframe['created'] = pd.to_datetime(dataframe['workflow_start_time'], utc=True)
        dataframe['resolutiondate'] = pd.to_datetime(dataframe['workflow_end_time'], utc=True)
        dataframe = dataframe.dropna(subset=['workflow_start_time', 'workflow_end_time'])
        return dataframe
    dataframe['created'] = pd.to_datetime(dataframe['created'], utc=True)
    dataframe['resolutiondate'] = pd.to_datetime(dataframe['resolutiondate'], utc=True)
    dataframe = dataframe.dropna(subset=['resolutiondate', 'created'])
    return dataframe


def genarate_creation_resolution_figure(dataframe):
    dataframe = str_df_to_dt_df(dataframe)
    dataframe.sort_values(by='created', inplace=True)

    plt.figure(figsize=(12, 6))
    plt.scatter(dataframe.index, dataframe['created'], label='Date de création', color='blue')
    plt.scatter(dataframe.index, dataframe['resolutiondate'], label='Date de résolution', color='green')
    plt.xlabel('ticket')
    plt.ylabel('dates creation_resolution')
    plt.title('Tendances creation_resolution des tickets ')
    plt.xticks(rotation=45)
    plt.legend()
    plt.tight_layout()
    plt.savefig(config.image_directory + 'creation_resolution_by_issue_type.png')
    plt.show()


def genarate_creation_resolution_figure_by_issue_field(dataframe, field):
    dataframe = str_df_to_dt_df(dataframe)

    dataframe.sort_values(by='created', inplace=True)

    types_issues = dataframe[field].unique()

    fig, axes = plt.subplots(nrows=len(types_issues), ncols=1, figsize=(12, 6 * len(types_issues)))

    for i, issue_type in enumerate(types_issues):
        data_filtered = dataframe[dataframe[field] == issue_type]

        axes[i].scatter(data_filtered.index, data_filtered['created'], label='Date de création', color='blue')

        axes[i].scatter(data_filtered.index, data_filtered['resolutiondate'], label='Date de résolution', color='green')

        axes[i].set_xlabel('ticket')
        axes[i].set_ylabel('dates creation_resolution')
        axes[i].set_title(f'Tendances creation_resolution des tickets ({issue_type})')
        axes[i].legend()
        axes[i].tick_params(axis='x', rotation=45)

    plt.tight_layout()
    plt.savefig(config.image_directory + f'/creation_resolution_by_issue_{field}.png')
    plt.show()


def get_creation_resolution_time_statistics(dataframe):
    dataframe = str_df_to_dt_df(dataframe)
    dataframe['creation_resolution_time'] = dataframe.apply(lambda row: row['resolutiondate'] - row['created'], axis=1)
    # Différence de temps moyenne entre création et résolution
    average_resolution_time = dataframe['creation_resolution_time'].mean()
    return dataframe['creation_resolution_time'].describe()


def get_month_statistics(dataframe):
    dataframe = str_df_to_dt_df(dataframe)
    dataframe['created_month'] = dataframe['created'].dt.month
    dataframe['resolution_time'] = dataframe['resolutiondate'] - dataframe['created']  #
    dataframe = dataframe[['created_month', 'resolution_time']]
    statistics_by_month = dataframe.groupby('created_month')['resolution_time'].describe()
    return statistics_by_month


def get_period_statistics(dataframe, period):
    dataframe = str_df_to_dt_df(dataframe)
    dataframe['period'] = get_data_by_period(dataframe, period)
    dataframe['resolution_time'] = dataframe['resolutiondate'] - dataframe['created']
    dataframe = dataframe[['period', 'resolution_time']]
    statistics_by_period = dataframe.groupby('period')['resolution_time'].describe()
    return statistics_by_period


# def get_statistics_by_issue_type(dataframe):
#     dataframe = str_df_to_dt_df(dataframe)
#     dataframe['resolution_time'] = dataframe['resolutiondate'] - dataframe['created']
#     statistics_by_issue_type = dataframe.groupby('issuetypename')['resolution_time'].describe()
#
#     return statistics_by_issue_type


def get_statistics_by_field(dataframe, field):
    dataframe = str_df_to_dt_df(dataframe)
    dataframe['resolution_time'] = dataframe['resolutiondate'] - dataframe['created']
    statistics_by_issue_field = dataframe.groupby(field)['resolution_time'].describe()
    return statistics_by_issue_field


def get_double_group_by_statistics(dataframe, period):
    dataframe = str_df_to_dt_df(dataframe)
    dataframe['resolution_time'] = dataframe['resolutiondate'] - dataframe['created']
    dataframe['period'] = get_data_by_period(dataframe, period)

    statistics = dataframe.groupby(['period', "issuetypename"])['resolution_time'].describe()
    return statistics


def get_data_by_period(dataframe, period):
    if period == 'week':
        return dataframe['created'].dt.isocalendar().week
    elif period == 'month':
        return dataframe['created'].dt.month
    elif period == 'year':
        return dataframe['created'].dt.year
    else:
        raise ValueError("Période non valide. Veuillez spécifier 'day', 'month' ou 'year'.")


def get_number_issues_solved_within_preriod_graph(dataframe, period):
    dataframe = str_df_to_dt_df(dataframe)
    dataframe['resolution_time_days'] = (dataframe['resolutiondate'] - dataframe['created']).dt.days
    print(dataframe['resolution_time_days'].describe())
    # 3. Identify and visualize the proportion of issues resolved within 30 days
    dataframe['resolved_within'] = dataframe['resolution_time_days'] <= period

    resolution_proportion = dataframe['resolved_within'].value_counts(normalize=True) * 100
    plt.figure(figsize=(6, 6))
    resolution_proportion.plot(kind='pie', autopct='%1.1f%%', startangle=140,
                               labels=[f'en plus de  {period} jour', f'en moins de  {period} jour'],
                               colors=['lightgreen', 'lightcoral'])
    plt.ylabel('')
    plt.title(f'Proportion des tickets resolu en  {period} jour')
    plt.tight_layout()
    plt.savefig(config.image_directory + f'issues_resolved_within_{period}_days.png')
    plt.show()


def plot_issue_types_distribution(df):
    issue_counts = df['issuetypename'].value_counts()
    creer_histogramme(issue_counts, 'Répartition des Types d\'Issues', 'Type d\'Issue', 'Nombre d\'Issues')


def analyze_time_spent_on_issues(df):
    df['timespend_hours'] = df['timespend'] / 3600  # Convertir le temps passé en heures
    print(df['timespend_hours'].describe())
    average_time = df.groupby('issuetypename')['timespend_hours'].mean()
    creer_histogramme(average_time, 'Temps Moyen Passé par Type d\'Issue', 'Type d\'Issue', 'Temps Moyen (Heures)')


def tickets_clotures_en_mois(dataframe, annee, mois):
    dataframe = str_df_to_dt_df(dataframe)
    dataframe_mois_annee = dataframe[
        (dataframe['resolutiondate'].dt.year == annee) & (dataframe['resolutiondate'].dt.month == mois)]

    nombre_tickets_clotures = dataframe_mois_annee.shape[0]
    return nombre_tickets_clotures


def tickets_clotures_par_mois(dataframe, annee):
    dataframe = str_df_to_dt_df(dataframe)
    dataframe_annee = dataframe[dataframe['resolutiondate'].dt.year == annee]

    tickets_clotures = dataframe_annee['resolutiondate'].dt.month.value_counts().sort_index()
    creer_courbe(Serie=tickets_clotures, title_graphique='Nombre de tickets clôturés par mois pour l\'année {annee}',
                 title_x='Mois', title_y='Nombre de tickets clôturés', xticks_rotation='Mois')


###
def lead_time_par_mois(dataframe):
    dataframe = str_df_to_dt_df(dataframe)

    # Calculer le lead time pour chaque ticket et l'ajouter comme une nouvelle colonne
    dataframe['lead_time'] = (dataframe['resolutiondate'] - dataframe['created']).dt.days

    years = dataframe['created'].dt.year.unique()

    for annee in years:
        # Filtrer les données pour l'année spécifiée
        dataframe_annee = dataframe[dataframe['created'].dt.year == annee]

        # Calculer la moyenne du lead time par mois
        lead_time_moyen = dataframe_annee.groupby(dataframe_annee['created'].dt.month)['lead_time'].mean()
        creer_courbe(Serie=lead_time_moyen, title_graphique=f'Lead time moyen par mois pour toutes les années',
                     title_x='Mois', title_y='Lead time moyen (jours)', xticks_rotation='Mois')
    return True


# Remplacer les deux fonctions suivantes par une seule
def tickets_cloturés_crées_par_annee(dataframe, field='resolutiondate'):
    dataframe = str_df_to_dt_df(dataframe)
    tickets_clotures = dataframe[dataframe[field].notnull()][field].dt.year.value_counts().sort_index()
    type_ticket = "créés" if field == 'created' else "cloturés"
    return creer_courbe(Serie=tickets_clotures, title_graphique=f'Nombre de tickets {type_ticket} par année',
                        title_x='Année', title_y=f'Nombre de tickets {type_ticket}')


###
def nombre_ticket_creer_cloture(dataframe):
    dataframe = str_df_to_dt_df(dataframe)

    tickets_crees = dataframe['created'].dt.year.value_counts().sort_index()
    tickets_clotures = dataframe[dataframe['resolutiondate'].notnull()][
        'resolutiondate'].dt.year.value_counts().sort_index()

    plt.figure(figsize=(10, 6))

    # courbe pour les tickets créés
    plt.plot(tickets_crees.index, tickets_crees.values, marker='o', label='Tickets créés')
    # courbe pour les tickets clôturés
    plt.plot(tickets_clotures.index, tickets_clotures.values, marker='o', label='Tickets clôturés')

    plt.xlabel('Année')
    plt.ylabel('Nombre de tickets')
    plt.title('Nombre de tickets créés et clôturés par année')
    plt.grid(True)
    plt.legend()
    filename = os.path.join(config.output_directory, 'nombre_de_tickets_cree_et_clôturés_par_année.png')
    plt.savefig(filename)
    # Afficher le graphique
    # plt.show()
    return 'nombre_de_tickets_cree_et_clôturés_par_année.png'


def tickets_cloturés_ou_créés_par_mois_par_annee(dataframe, field='resolutiondate'):
    dataframe = str_df_to_dt_df(dataframe)
    annees = dataframe[field].dt.year.unique()
    liste_courbes = []
    type_ticket = "créés" if field == 'created' else "cloturés"
    for annee in annees:
        dataframe_annee = dataframe[dataframe[field].dt.year == annee]
        tickets_clotures_par_mois = dataframe_annee.groupby(dataframe_annee[field].dt.month).size()
        liste_courbes.append(
            creer_courbe(tickets_clotures_par_mois, f'Nombre de tickets {type_ticket} pour l\'année {annee}',
                         'Mois', f'Nombre de tickets {type_ticket}', 'Mois'))
    return liste_courbes


def lead_time_par_annee(dataframe):
    dataframe = str_df_to_dt_df(dataframe)
    dataframe['lead_time'] = (dataframe['resolutiondate'] - dataframe['created']).dt.days
    lead_time_moyen = dataframe.groupby(dataframe['created'].dt.year)['lead_time'].mean()

    return creer_courbe(Serie=lead_time_moyen, title_graphique='Lead time moyen par année', title_x='Année',
                        title_y='Lead time moyen (jours)')


def tickets_status_par_intervals_de_temps(status, dataframe, date_min, date_max):
    dataframe = str_df_to_dt_df(dataframe)
    date_resolution = dataframe[status]

    date_min = pd.Timestamp(date_min)
    date_max = pd.Timestamp(date_max)

    dataframe_intervals = dataframe[
        (date_resolution.dt.date >= date_min.date()) & (date_resolution.dt.date <= date_max.date())]

    tickets_clotures_par_date = dataframe_intervals.groupby(dataframe_intervals[status].dt.date).size()

    return creer_courbe(tickets_clotures_par_date, f'Nombre de tickets clôturés du {date_min.date()} '
                                                   f'au {date_max.date()}', 'Date',
                        'Nombre de tickets clôturés', 40)


def lead_time_par_intervals_de_temps(dataframe, date_min, date_max):
    dataframe = str_df_to_dt_df(dataframe)

    date_min = pd.Timestamp(date_min)
    date_max = pd.Timestamp(date_max)

    date_created = dataframe['created']
    date_resolution = dataframe['resolutiondate']

    # Data frame de notre intervals de temps
    dataframe_intervals = dataframe[
        (date_created.dt.date >= date_min.date()) & (date_resolution.dt.date <= date_max.date())]

    # Calculer le lead time pour chaque ticket et l'ajouter comme une nouvelle colonne
    dataframe_intervals['lead_time'] = (
            dataframe_intervals['resolutiondate'] - dataframe_intervals['created']).dt.days

    lead_time_moyen = dataframe_intervals.groupby(dataframe_intervals['created'].dt.date)['lead_time'].mean()

    # Créer le graphique
    return creer_courbe(lead_time_moyen,
                        f'Lead time moyen par intervals du {date_min.date()} au {date_max.date()}',
                        'Date', 'Lead time moyen ', 40)


def creer_courbe(Serie, title_graphique='', title_x='', title_y='', xticks_rotation=40):
    if not os.path.exists(config.output_directory):
        os.makedirs(config.output_directory)
    plt.figure(figsize=(10, 6))
    if Serie is not None:
        plt.plot(Serie.index, Serie.values, marker='o')
    plt.xlabel(title_x)
    plt.ylabel(title_y)
    plt.title(title_graphique)
    plt.grid(True)
    if isinstance(xticks_rotation, int) is False:
        plt.xticks(range(1, 13),
                   ['Jan', 'Fév', 'Mar', 'Avr', 'Mai', 'Juin', 'Juil', 'Août', 'Sep', 'Oct', 'Nov', 'Déc'])
    else:
        plt.xticks(rotation=xticks_rotation)
    plt.tight_layout()
    filename = os.path.join(config.output_directory, f'{title_graphique}.png')
    plt.savefig(filename)
    plt.grid(True)
    plt.show()
    return f'{title_graphique}.png'


def creer_histogramme(values, title, x_title, y_title):
    if not os.path.exists(config.output_directory):
        os.makedirs(config.output_directory)
    values.plot(kind='bar')
    plt.title(title)
    plt.xlabel(x_title)
    plt.ylabel(y_title)
    plt.xticks(rotation=12)
    filename = os.path.join(config.output_directory, f'{title}.png')
    plt.savefig(filename)
    plt.show()


def lead_time_moyen_par_mois_par_annee_pour_nombre_annee(dataframe, nb_dernieres_annees):
    dataframe = str_df_to_dt_df(dataframe)
    annees = dataframe['created'].dt.year.unique()
    dernieres_annees = sorted(annees)[-nb_dernieres_annees:]

    for annee in dernieres_annees:
        dataframe_annee = dataframe[dataframe['created'].dt.year == annee]
        dataframe_annee['lead_time'] = (dataframe_annee['resolutiondate'] - dataframe_annee['created']).dt.days
        lead_time_moyen_par_mois = dataframe_annee.groupby(dataframe_annee['created'].dt.month)['lead_time'].mean()
        plt.plot(lead_time_moyen_par_mois.index, lead_time_moyen_par_mois.values, marker='o', label=f'Année {annee}')
    return creer_courbe(None, f'Lead time moyen par mois pour les {nb_dernieres_annees} dernières années',
                        'Mois', 'Lead time moyen (jours)', 'Mois')


def lead_time_moyen_par_mois_par_annee_pour_deux_annee(dataframe, annee1, annee2):
    dataframe = str_df_to_dt_df(dataframe)
    annees = [annee1, annee2]

    for annee in annees:
        dataframe_annee = dataframe[dataframe['created'].dt.year == annee]
        dataframe_annee['lead_time'] = (dataframe_annee['resolutiondate'] - dataframe_annee['created']).dt.days
        lead_time_moyen_par_mois = dataframe_annee.groupby(dataframe_annee['created'].dt.month)['lead_time'].mean()
        plt.plot(lead_time_moyen_par_mois.index, lead_time_moyen_par_mois.values, marker='o', label=f'Année {annee}')
    return creer_courbe(None, f'Lead time moyen par mois pour les {annee1} et {annee2}',
                        'Mois', 'Lead time moyen (jours)', 'Mois')


def tickets_status_par_mois_par_annee_pour_nombre_annee(status, dataframe, nb_dernieres_annees):
    dataframe = str_df_to_dt_df(dataframe)
    annees = dataframe[status].dt.year.unique()
    list_courbes = []
    dernieres_annees = sorted(annees)[-nb_dernieres_annees:]
    for annee in dernieres_annees:
        dataframe_annee = dataframe[dataframe[status].dt.year == annee]
        tickets_crees_par_mois = dataframe_annee.groupby(dataframe_annee[status].dt.month).size()

        plt.plot(tickets_crees_par_mois.index, tickets_crees_par_mois.values, marker='o', label=f'Année {annee}')
        type_ticket = "créés" if status == 'created' else "cloturés"

    list_courbes.append(
        creer_courbe(None, f'Nombre de tickets {type_ticket} par mois pour les {nb_dernieres_annees} dernières années',
                     'Mois', f'Nombre de tickets {type_ticket}', 'Mois'))

    return list_courbes


#
# def tickets_crees_par_mois_par_annee_pour_nombre_annee(dataframe, nb_dernieres_annees):
#     dataframe = str_df_to_dt_df(dataframe)
#     annees = dataframe['created'].dt.year.unique()
#
#     dernieres_annees = sorted(annees)[-nb_dernieres_annees:]
#     for annee in dernieres_annees:
#         dataframe_annee = dataframe[dataframe['created'].dt.year == annee]
#         tickets_crees_par_mois = dataframe_annee.groupby(dataframe_annee['created'].dt.month).size()
#
#         plt.plot(tickets_crees_par_mois.index, tickets_crees_par_mois.values, marker='o', label=f'Année {annee}')
#     creer_courbe(None, f'Nombre de tickets créés par mois pour les {nb_dernieres_annees} dernières années',
#                  'Mois', 'Nombre de tickets créés', 'Mois')
#
#     return True
#
#
# def tickets_clotures_par_mois_par_annee_pour_nombre_annee(dataframe, nb_dernieres_annees):
#     dataframe = str_df_to_dt_df(dataframe)
#     annees = dataframe['resolutiondate'].dt.year.unique()
#     dernieres_annees = sorted(annees)[-nb_dernieres_annees:]
#
#     for annee in dernieres_annees:
#         dataframe_annee = dataframe[dataframe['resolutiondate'].dt.year == annee]
#         tickets_clotures_par_mois = dataframe_annee.groupby(dataframe_annee['resolutiondate'].dt.month).size()
#
#         plt.plot(tickets_clotures_par_mois.index, tickets_clotures_par_mois.values, marker='o', label=f'Année {annee}')
#     creer_courbe(None, f'Nombre de tickets clôturés par mois pour les {nb_dernieres_annees} dernières années',
#                  'Mois', 'Nombre de tickets clôturés', 'Mois')
#     return True


def lead_time_par_mois_par_annee(dataframe):
    dataframe = str_df_to_dt_df(dataframe)
    annees = dataframe['resolutiondate'].dt.year.unique()
    list_courbes = []
    for annee in annees:
        dataframe_annee = dataframe[dataframe['resolutiondate'].dt.year == annee]
        tickets_clotures_par_mois = dataframe_annee.groupby(dataframe_annee['resolutiondate'].dt.month).size()
        return creer_courbe(tickets_clotures_par_mois,
                            f'Nombre de tickets clôturés par mois pour l\'andatetimenée {annee}',
                            'Mois', 'Nombre de tickets clôturés', 'Mois')



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    # Here is the module name.
    name="synch2jira",
    # version of the module
    version="0.0.208",
    # Name of Author
    author="wefine",
    # your Email address
    author_email="wefine2529@ebuthor.com",
    # #Small Description about module
    description="Lib",

    long_description=long_description,
    long_description_content_type="text/markdown",

    packages=setuptools.find_packages(),

    install_requires=[
        "requests==2.31.0",
        "setuptools==68.2.0",
        "SQLAlchemy==2.0.25",
        "pandas",
    ],

    license="MIT",
    # classifiers like program is suitable for python3, just leave as it is.
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)
